import * as React from "react";
import { IFormBinder, IDataBinder, IFormBinderInjector } from "./formCore";
import { IValueConverter, IInputValueConverter, INumericOptions } from "./formValueConverters";
import { FormBinderBase } from "./formBinderBase";
import { ICalendarInputProps } from "./inputs/calendarInput";
import { IDateInputProps } from "./inputs/dateInput";
import { ITimeInputProps } from "./inputs/timeInput";
import { IAutoCompleteInputProps } from "./inputs/autoCompleteInput";
/** An input FormBinder that sets native 'value' and 'onChange: (e) => void' properties */
export declare class InputFormBinder<TDataPropValue, TComponentPropValue> extends FormBinderBase<React.DOMAttributes<{}>, TDataPropValue, TComponentPropValue> {
    setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>): void;
    protected getDefaultInputValue(): any;
    handleValueChanged(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class CheckboxFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue> {
    protected getDefaultInputValue(): boolean;
}
/** A radio input FormBinder */
export declare class RadioFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue> {
    setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>): void;
}
export declare class DateInputFormBinder extends FormBinderBase<IDateInputProps, string, string> {
    constructor(dataName: string);
    handleValueChanged(props: IDateInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class TimeInputFormBinder extends FormBinderBase<ITimeInputProps, string, string> {
    constructor(dataName: string);
    static customValue(dataName: string): TimeInputFormBinder;
    handleValueChanged(props: ITimeInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class CalendarInputFormBinder extends FormBinderBase<ICalendarInputProps, string, string> {
    constructor(dataName: string);
    static customValue(dataName: string): CalendarInputFormBinder;
    handleValueChanged(props: ICalendarInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class AutoCompleteFormBinder implements IFormBinder<IAutoCompleteInputProps, any> {
    private dataPath;
    constructor(dataPath: string);
    setElementProperty(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>): void;
    handleValueChanged(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
/** Form Binder helpers */
export declare class FormBinder {
    static custom<P>(formBinder: IFormBinder<P, any>): IFormBinderInjector<P>;
    /** hidden */
    static hidden<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** password */
    static password<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** text */
    static text<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** text 'email' */
    static textEmail<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    static autoCompleteInput(dataName: string): IFormBinderInjector<IAutoCompleteInputProps>;
    static calendarInput(dataName: string): IFormBinderInjector<ICalendarInputProps>;
    static dateInput(dataName: string): IFormBinderInjector<IDateInputProps>;
    static timeInput(dataName: string): IFormBinderInjector<ITimeInputProps>;
    private static defaultInputFormBinder<TDataPropValue, TTo>(dataName, type, valueConverter?, propertySet?);
    /** range */
    static range(dataName: string, options?: INumericOptions): IFormBinderInjector<React.DOMAttributes<any>>;
    /** uncontroller text input */
    static defaultText<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** numeric text */
    static textNumeric(dataName: string, options?: INumericOptions): IFormBinderInjector<React.DOMAttributes<any>>;
    /** generic select */
    static selectCustom<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** select string */
    static select(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** select number */
    static selectNumeric(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** generic checkbox */
    static checkboxCustom<TDataPropValue>(/** helooo */ dataName: string, valueConverter?: IValueConverter<TDataPropValue, boolean>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** checkbox binding to boolean */
    static checkbox(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** checkbox binding to boolean derived from trueValue and falseValue equality testing */
    static checkboxConvert<TDataPropValue>(dataName: string, trueValue: TDataPropValue, falseValue: TDataPropValue): IFormBinderInjector<React.DOMAttributes<any>>;
    /** generic input 'radio' */
    static radioCustom<TDataPropValue>(dataName: string, value: string, valueConverter: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** input 'radio' string value */
    static radio(dataName: string, value: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** input 'radio' numeric value */
    static radioNumeric(dataName: string, value: number): IFormBinderInjector<React.DOMAttributes<any>>;
}
